@@ -132,7 +132,7 @@ class SchemaParser(object):
                 pp.delimitedList(pp.Group(ident + pp.Optional(colon) + pp.Group(type_expr)))) + r_angle
 
             # try to capture invalid type name for better error reporting
-            invalid_type = pp.Word(pp.alphas, pp.alphanums+"_", as_keyword=True)
+            invalid_type = pp.Word(pp.alphas, pp.alphanums+"_", asKeyword=True)
 
             # use left recursion to handle nesting of types
             type_expr <<= pp.MatchFirst([primitive_type_keyword, array_expr, map_expr, struct_expr, invalid_type])
@@ -260,6 +260,68 @@ class SchemaParser(object):
 
         return type_construct
 
+    @classmethod
+    def _cleanseSQL(cls, sql_string):
+        """ Cleanse sql string removing string literals so that they are not considered as part of potential column
+            references
+        :param sql_string: String representation of SQL expression
+        :returns: cleansed string
+
+        Any strings identified are replaced with `' '`
+        """
+        assert sql_string is not None, "`sql_string` must be specified"
+
+        # skip over quoted identifiers even if they contain quotes
+        quoted_ident = pp.QuotedString(quoteChar="`", escQuote="``")
+        quoted_ident.setParseAction(lambda s, loc, toks: f"`{toks[0]}`")
+
+        stringForm1 = pp.Literal('r') + pp.QuotedString(quoteChar="'")
+        stringForm2 = pp.Literal('r') + pp.QuotedString(quoteChar='"')
+        stringForm3 = pp.QuotedString(quoteChar="'", escQuote=r"\'")
+        stringForm4 = pp.QuotedString(quoteChar='"', escQuote=r'\"')
+        stringForm = stringForm1 ^ stringForm2 ^ stringForm3 ^ stringForm4
+        stringForm.setParseAction(lambda s, loc, toks: "' '")
+
+        parser = quoted_ident ^ stringForm
+
+        transformed_string = parser.transformString(sql_string)
+
+        return transformed_string
+
+    @classmethod
+    def columnsReferencesFromSQLString(cls, sql_string, filter=None):
+        """ Generate a list of possible column references from a SQL string
+
+        This method finds all condidate references to SQL columnn ids in the string
+
+        To avoid the overhead of a full SQL parser, the implementation will simply look for possible field names
+
+        Further improvements may eliminate some common syntax but in current form, reserved words will
+        also be returned as possible column references.
+
+        So any uses of this must not assume that all possible references are valid column references
+
+        :param sql_string: String representation of SQL expression
+        :returns: list of possible column references
+        """
+        assert sql_string is not None, "`sql_string` must be specified"
+        assert filter is None or isinstance(filter, list) or isinstance(filter, set)
+
+        cleansed_sql_string = cls._cleanseSQL(sql_string)
+
+        ident = pp.Word(pp.alphas, pp.alphanums + "_") | pp.QuotedString(quoteChar="`", escQuote="``")
+        parser = ident
+
+        references = parser.searchString(cleansed_sql_string)
+
+        results = set([item for sublist in references for item in sublist])
+
+        if filter is not None:
+            filtered_results = results.intersection(set(filter))
+            return list(filtered_results)
+        else:
+            return list(results)
+
     @classmethod
     def parseCreateTable(cls, sparkSession, source_schema):
         """ Parse a schema from a schema string
